import errno
import logging
import os
import unittest
import requests

import wildfly.util
from tests.integration import base
from wildfly.api.deployment import DEFAULT_SERVER_GROUP

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)

DEFAULT_NEXUS_HOST = 'nexus'
DEFAULT_NEXUS_PORT = '8081'
DEFAULT_GROUP_ID = 'com'
DEFAULT_ARTIFACT_ID = 'project'
DEFAULT_ARTIFACT_VERSION = '1.0'
DEFAULT_ARTIFACT_TYPE = 'jar'
DEFAULT_DEPLOYMENT_NAME = '{}-{}.{}'.format(
    DEFAULT_ARTIFACT_ID,
    DEFAULT_ARTIFACT_VERSION,
    DEFAULT_ARTIFACT_TYPE)


class DeploymentTest(base.BaseTestCase):

    def test_deployments_with_opitions(self):
        try:
            self.client.deploy(
                DEFAULT_GROUP_ID,
                DEFAULT_ARTIFACT_ID,
                DEFAULT_ARTIFACT_VERSION,
                type='jar'
            )
            deployments = self.client.deployments(
                server_group=DEFAULT_SERVER_GROUP)
            for key in deployments.keys():
                self.assertIn(
                    DEFAULT_SERVER_GROUP,
                    deployments[key]['server-groups'])
        except Exception as e:
            self.fail(base.UNEXPECTED_EXCEPTION_FORMAT.format(e))
        finally:
            self.client.undeploy(DEFAULT_DEPLOYMENT_NAME)

    # TODO: ADD Test to cover option: host. Currently, host name is docker
    # instance ip in fixture setup.

    def test_list_deployments(self):
        try:
            self.client.deploy(
                DEFAULT_GROUP_ID,
                DEFAULT_ARTIFACT_ID,
                DEFAULT_ARTIFACT_VERSION,
                type='jar'
            )
            deployments = self.client.deployments()
            self.assertIn(DEFAULT_DEPLOYMENT_NAME, deployments)
            self.assertTrue(
                self.client.is_deployment_enabled(DEFAULT_DEPLOYMENT_NAME))
            self.assertEqual(self.client.deployment_status(
                DEFAULT_DEPLOYMENT_NAME), 'RUNNING')
        except Exception as e:
            self.fail(base.UNEXPECTED_EXCEPTION_FORMAT.format(e))
        finally:
            self.client.undeploy(DEFAULT_DEPLOYMENT_NAME)

    def test_list_deployments_disabled(self):
        try:
            self.client.deploy(
                DEFAULT_GROUP_ID,
                DEFAULT_ARTIFACT_ID,
                DEFAULT_ARTIFACT_VERSION,
                type='jar',
                enabled=False
            )
            deployments = self.client.deployments()
            self.assertIn(DEFAULT_DEPLOYMENT_NAME, deployments)
            self.assertFalse(
                self.client.is_deployment_enabled(DEFAULT_DEPLOYMENT_NAME))
            self.assertEqual(self.client.deployment_status(
                DEFAULT_DEPLOYMENT_NAME), 'STOPPED')
        except Exception as e:
            self.fail(base.UNEXPECTED_EXCEPTION_FORMAT.format(e))
        finally:
            self.client.undeploy(DEFAULT_DEPLOYMENT_NAME)

    def test_list_deployments_server_down(self):
        try:
            self.client.stop_servers(blocking=True)
            self.client.deploy(
                DEFAULT_GROUP_ID,
                DEFAULT_ARTIFACT_ID,
                DEFAULT_ARTIFACT_VERSION,
                type='jar'
            )
            deployments = self.client.deployments()
            self.assertIn(DEFAULT_DEPLOYMENT_NAME, deployments)
            self.assertTrue(deployments[DEFAULT_DEPLOYMENT_NAME]['enabled'])
            self.assertEqual(
                deployments[DEFAULT_DEPLOYMENT_NAME]['status'],
                'STOPPED')
        except Exception as e:
            self.fail(base.UNEXPECTED_EXCEPTION_FORMAT.format(e))
        finally:
            self.client.undeploy(DEFAULT_DEPLOYMENT_NAME)
            self.client.start_servers(blocking=True)

    def test_list_deployments_in_group(self):
        try:
            self.client.deploy(
                DEFAULT_GROUP_ID,
                DEFAULT_ARTIFACT_ID,
                DEFAULT_ARTIFACT_VERSION,
                type='jar'
            )
            deployments = self.client.deployments(
                server_group=DEFAULT_SERVER_GROUP)
        except Exception as e:
            self.fail(base.UNEXPECTED_EXCEPTION_FORMAT.format(e))
        finally:
            self.client.undeploy(DEFAULT_DEPLOYMENT_NAME)
        self.assertIn(DEFAULT_DEPLOYMENT_NAME, deployments)

    def test_is_deployment_in_repository(self):
        try:
            self.client.deploy(
                DEFAULT_GROUP_ID,
                DEFAULT_ARTIFACT_ID,
                DEFAULT_ARTIFACT_VERSION)
            result = self.client.is_deployment_in_repository(
                DEFAULT_DEPLOYMENT_NAME)
            self.assertTrue(result)
        except Exception as e:
            self.fail(base.UNEXPECTED_EXCEPTION_FORMAT.format(e))
        finally:
            self.client.undeploy(DEFAULT_DEPLOYMENT_NAME)

    def test_is_deployment_not_in_repository(self):
        try:
            result = self.client.is_deployment_in_repository(
                DEFAULT_DEPLOYMENT_NAME)
            self.assertFalse(result)
        except Exception as e:
            self.fail(base.UNEXPECTED_EXCEPTION_FORMAT.format(e))

    def test_is_deployment_enabled(self):
        try:
            self.client.deploy(
                DEFAULT_GROUP_ID,
                DEFAULT_ARTIFACT_ID,
                DEFAULT_ARTIFACT_VERSION)
            result = self.client.is_deployment_enabled(DEFAULT_DEPLOYMENT_NAME)
            self.assertTrue(result)
        except Exception as e:
            self.fail(base.UNEXPECTED_EXCEPTION_FORMAT.format(e))
        finally:
            self.client.undeploy(DEFAULT_DEPLOYMENT_NAME)


class PullTest(base.BaseTestCase):

    def test_pull(self):
        try:
            self.client.pull(
                DEFAULT_GROUP_ID,
                DEFAULT_ARTIFACT_ID,
                DEFAULT_ARTIFACT_VERSION,
                type='jar')
            deployments = self.client.deployments()
            self.assertIn(DEFAULT_DEPLOYMENT_NAME, deployments)
            self.assertFalse(deployments[DEFAULT_DEPLOYMENT_NAME]['enabled'])
            self.assertEqual(
                deployments[DEFAULT_DEPLOYMENT_NAME]['status'],
                'STOPPED')
        except Exception as e:
            self.fail(base.UNEXPECTED_EXCEPTION_FORMAT.format(e))
        finally:
            self.client.undeploy(DEFAULT_DEPLOYMENT_NAME)


class DeployTest(base.BaseTestCase):

    def test_deploy_url(self):

        try:
            self.client.deploy(
                DEFAULT_GROUP_ID,
                DEFAULT_ARTIFACT_ID,
                DEFAULT_ARTIFACT_VERSION)
            deployments = self.client.deployments()
            self.assertIn(DEFAULT_DEPLOYMENT_NAME, deployments)
            self.assertTrue(deployments[DEFAULT_DEPLOYMENT_NAME]['enabled'])
            self.assertEqual(
                deployments[DEFAULT_DEPLOYMENT_NAME]['status'],
                'RUNNING')
        except Exception as e:
            self.fail(base.UNEXPECTED_EXCEPTION_FORMAT.format(e))
        finally:
            self.client.undeploy(DEFAULT_DEPLOYMENT_NAME)

    def test_deploy_file(self):

        NEXUS_BASE_URL = 'http://{}:{}' \
                         '/service/local/repositories/releases/content'\
            .format(DEFAULT_NEXUS_HOST, DEFAULT_NEXUS_PORT)

        url = '{0}/{1}/{2}/{3}/{2}-{3}.{4}'.format(
            NEXUS_BASE_URL,
            DEFAULT_GROUP_ID.replace('.', '/'),
            DEFAULT_ARTIFACT_ID,
            DEFAULT_ARTIFACT_VERSION,
            DEFAULT_ARTIFACT_TYPE)
            
        response = requests.get(url)

        if response.status_code not in [200, 204]:
            print(
                'Response Status Code: {}: {}'.format(
                    response.status_code,
                    response.reason))
            response.raise_for_status()

        filename = '{}-{}.{}'.format(DEFAULT_ARTIFACT_ID,
                                     DEFAULT_ARTIFACT_VERSION,
                                     DEFAULT_ARTIFACT_TYPE)

        with open(filename, "wb") as artifact:
            artifact.write(response.content)

        try:
            self.client.deploy(
                DEFAULT_GROUP_ID,
                DEFAULT_ARTIFACT_ID,
                DEFAULT_ARTIFACT_VERSION,
                path=filename)
            deployments = self.client.deployments()
            self.assertIn(DEFAULT_DEPLOYMENT_NAME, deployments)
            self.assertTrue(deployments[DEFAULT_DEPLOYMENT_NAME]['enabled'])
            self.assertEqual(
                deployments[DEFAULT_DEPLOYMENT_NAME]['status'],
                'RUNNING')
        except Exception as e:
            self.fail(base.UNEXPECTED_EXCEPTION_FORMAT.format(e))
        finally:
            self.client.undeploy(DEFAULT_DEPLOYMENT_NAME)

        os.remove(filename)

    def test_deploy_file_not_exist(self):

        with self.assertRaises(IOError) as cm:
            self.client.deploy(
                DEFAULT_GROUP_ID,
                DEFAULT_ARTIFACT_ID,
                DEFAULT_ARTIFACT_VERSION,
                path='no-file-at-path')
        exception = cm.exception
        self.assertEqual(exception.errno, errno.ENOENT)
        self.assertEqual(exception.strerror, os.strerror(exception.errno))

    def test_deploy_url_not_exist(self):

        with self.assertRaisesRegexp(requests.exceptions.HTTPError,
                                     "404 Client Error: Not Found"):
            self.client.deploy('fake', 'fake', 'fake')

    @unittest.skip("still working on this one")
    def test_redeploy_url(self):

        pass
        try:
            self.client.deploy(
                DEFAULT_GROUP_ID,
                DEFAULT_ARTIFACT_ID,
                DEFAULT_ARTIFACT_VERSION)
        except Exception as e:
            self.fail(base.UNEXPECTED_EXCEPTION_FORMAT.format(e))

        result = self.client.read_attribute(
            'enabled',
            [{'server-group': DEFAULT_SERVER_GROUP},
             {'deployment': DEFAULT_DEPLOYMENT_NAME}])

        self.assertTrue(result)

        try:
            self.client.deploy(
                DEFAULT_GROUP_ID,
                DEFAULT_ARTIFACT_ID,
                DEFAULT_ARTIFACT_VERSION)
        except Exception as e:
            self.fail(base.UNEXPECTED_EXCEPTION_FORMAT.format(e))

        result = self.client.read_attribute(
            'enabled',
            [{'server-group': DEFAULT_SERVER_GROUP},
             {'deployment': DEFAULT_DEPLOYMENT_NAME}])

        self.assertTrue(result)
        self.client.undeploy(DEFAULT_DEPLOYMENT_NAME)


class UndeployTest(base.BaseTestCase):

    def test_undeploy(self):

        try:
            self.client.deploy(
                DEFAULT_GROUP_ID,
                DEFAULT_ARTIFACT_ID,
                DEFAULT_ARTIFACT_VERSION)
            self.client.undeploy(DEFAULT_DEPLOYMENT_NAME)
            deployments = self.client.deployments()
            self.assertNotIn(DEFAULT_DEPLOYMENT_NAME, deployments)
        except Exception as e:
            self.fail(base.UNEXPECTED_EXCEPTION_FORMAT.format(e))

    def test_undeploy_not_deployed(self):

        response = self.client.undeploy(DEFAULT_DEPLOYMENT_NAME)
        self.assertFalse(wildfly.util.is_success(response))
        failure_description = response.json(
        )['failure-description']['domain-failure-description']
        self.assertIn('not found', failure_description)


class EnableDisableTest(base.BaseTestCase):

    def test_enable(self):
        try:
            self.client.pull(
                DEFAULT_GROUP_ID,
                DEFAULT_ARTIFACT_ID,
                DEFAULT_ARTIFACT_VERSION,
                type='jar')
            self.client.enable(DEFAULT_DEPLOYMENT_NAME, DEFAULT_SERVER_GROUP)
            self.assertTrue(
                self.client.is_deployment_enabled(DEFAULT_DEPLOYMENT_NAME))
        finally:
            self.client.undeploy(DEFAULT_DEPLOYMENT_NAME)

    def test_disable(self):
        try:
            self.client.deploy(
                DEFAULT_GROUP_ID,
                DEFAULT_ARTIFACT_ID,
                DEFAULT_ARTIFACT_VERSION)
            self.client.disable(DEFAULT_DEPLOYMENT_NAME, DEFAULT_SERVER_GROUP)
            self.assertFalse(
                self.client.is_deployment_enabled(DEFAULT_DEPLOYMENT_NAME))
        finally:
            self.client.undeploy(DEFAULT_DEPLOYMENT_NAME)


from wildfly.api.host import HostApiMixin
from .. import base

try:
    from unittest import mock
except ImportError:
    import mock


class HostApiMixinTest(base.BaseTestCase):

	def test_servers(self):
		pass

	def test_server_groups(self):
		pass

	def test_server_operation(self):
		pass

	def test_start_servers(self):
		pass

	def test_stop_servers(self):
		pass

	def test_reload_servers(self):
		pass

	def test_restart_servers(self):
		pass

	def test_read_log_file(self):
		pass






import time
import unittest

import requests

import base
import wildfly
import wildfly.util
from wildfly.api.deployment import DEFAULT_SERVER_GROUP


class ConnectionTimeoutTest(unittest.TestCase):

    def setUp(self):
        self.timeout = 0.5
        self.client = wildfly.Client(
            base.WILDFLY_CONTAINER_NAME,
            timeout=self.timeout)

    def tearDown(self):
        self.client.close()

    def test_timeout(self):
        start = time.time()
        response = None
        # This call isn't supposed to complete, and it should fail fast.
        try:
            operation = 'read-resource-fake'
            response = self.client.execute(operation=operation)
        except:
            pass
        end = time.time()
        self.assertTrue((response.status_code == 500))
        self.assertTrue(end - start < 2 * self.timeout)


class VersionTest(base.BaseTestCase):

    def test_version(self):
        try:
            result = self.client.version()
        except Exception as e:
            self.fail(base.UNEXPECTED_EXCEPTION_FORMAT.format(e))
        self.assertEqual(result, base.DEFAULT_WILDFLY_VERSION)


class ExecuteTest(base.BaseTestCase):

    def test_execute_defaults(self):
        operation = 'read-resource'
        response = self.client.execute(operation=operation)
        self.assertTrue(wildfly.util.is_success(response))

    def test_execute_address(self):
        operation = 'read-resource'
        address = [{'server-group': DEFAULT_SERVER_GROUP}]
        response = self.client.execute(operation=operation, address=address)
        self.assertTrue(wildfly.util.is_success(response))

    def test_execute_parms(self):
        operation = 'read-children-resources'
        parameters = {'child-type': 'host', 'include-runtime': 'true'}
        response = self.client.execute(
            operation=operation, parameters=parameters)
        self.assertTrue(wildfly.util.is_success(response))

    def test_execute_controller_bad_port(self):
        client = wildfly.Client(base.WILDFLY_CONTAINER_NAME, 99990)
        operation = 'read-resource'
        with self.assertRaises(requests.ConnectionError):
            client.execute(operation=operation)
        client.close()

    def test_execute_controller_bad_address(self):
        client = wildfly.Client('invalidaddress')
        operation = 'read-resource'
        with self.assertRaises(requests.ConnectionError):
            client.execute(operation=operation)
        client.close()

    @unittest.skip("TOO SLOW")
    def test_execute_controller_down(self):
        base.stop_wildfly_service()
        operation = 'read-resource'
        with self.assertRaisesRegexp(requests.ConnectionError,
                                     'No route to host'):
            self.client.execute(operation=operation)
        base.start_wildfly_service()


class AddTest(base.BaseTestCase):

    def test_add(self):

        address = [{'system-property': 'dummy'}]
        parameters = {'value': 'test'}
        try:
            response = self.client.add(address=address, parameters=parameters)
            self.assertTrue(wildfly.util.is_success(response))
            result = self.client.read_attribute(address=address, name='value')
            self.assertEqual(result, 'test')
        except Exception as e:
            self.fail(base.UNEXPECTED_EXCEPTION_FORMAT.format(e))
        finally:
            self.client.remove(address=address)

    def test_add_exists(self):
        address = [{'system-property': 'dummy'}]
        parameters = {'value': 'test'}
        try:
            response = self.client.add(address, parameters)
            self.assertTrue(wildfly.util.is_success(response))
            result = self.client.read_attribute(address=address, name='value')
            self.assertEqual(result, 'test')
            response = self.client.add(address, parameters)
            self.assertFalse(wildfly.util.is_success(response))
            failure_description = response.json(
            )['failure-description']['domain-failure-description']
            self.assertIn('Duplicate resource', failure_description)
        except Exception as e:
            self.fail(base.UNEXPECTED_EXCEPTION_FORMAT.format(e))
        finally:
            self.client.remove(address)


@unittest.skip("TODO test remove operation")
class RemoveTest(base.BaseTestCase):

    def test_remove(self):
        pass


class ReadResourceTest(base.BaseTestCase):

    def test_read_resource_default(self):
        result = self.client.read_resource()
        self.assertTrue(wildfly.util.is_success(result))

    def test_read_resource(self):
        address = [{'server-group': DEFAULT_SERVER_GROUP}]
        result = self.client.read_resource(address)
        self.assertTrue(wildfly.util.is_success(result))


class ReadAttributeTest(base.BaseTestCase):

    def test_read_attribute(self):
        result = self.client.read_attribute(address=[], name='process-type')
        self.assertEqual(result, 'Domain Controller')


class WriteAttributeTest(base.BaseTestCase):

    def test_write_attribute(self):
        current_value = self.client.read_attribute(address=[], name='name')
        result = self.client.write_attribute(
            address=[], name='name', value='test')
        result = self.client.read_attribute(address=[], name='name')
        self.assertEqual(result, 'test')
        self.client.write_attribute(
            address=[], name='name', value=current_value)


class ReadChildrenNamesTest(base.BaseTestCase):

    def test_read_children_names(self):
        result = self.client.read_children_names(
            address=[], child_type='server-group')
        self.assertIsNotNone(result)
        self.assertIn(DEFAULT_SERVER_GROUP, result)


class ReadChildrenResourcesTest(base.BaseTestCase):

    def test_read_children_resources(self):
        result = self.client.read_children_resources(
            address=[],
            child_type='server-group')
        self.assertIsNotNone(result)
        self.assertIn(DEFAULT_SERVER_GROUP, result)
        self.assertEqual(result[DEFAULT_SERVER_GROUP]['deployment'], None)



import time
import socket
import unittest
import docker
import wildfly

DEFAULT_WILDFLY_VERSION = '8.2.0.Final'
WILDFLY_CONTAINER_NAME = 'wildfly'
UNEXPECTED_EXCEPTION_FORMAT = 'Unexpected Exception: {}.'


def wait_for_wildfly_service():
    retries = 180
    delay = 1
    while retries > 0:
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            s.settimeout(1)
            s.connect((WILDFLY_CONTAINER_NAME, 9990))
            s.close()
            retries = 0
        except Exception:
            retries -= 1
        if retries > 0:
            time.sleep(delay)


def start_wildfly_service():
    docker_client = docker.Client(base_url='unix://var/run/docker.sock')
    docker_client.start(WILDFLY_CONTAINER_NAME)
    wait_for_wildfly_service()


def stop_wildfly_service():
    docker_client = docker.Client(base_url='unix://var/run/docker.sock')
    docker_client.stop(WILDFLY_CONTAINER_NAME)


class BaseTestCase(unittest.TestCase):

    def setUp(self):
        self.client = wildfly.Client(WILDFLY_CONTAINER_NAME)

    def tearDown(self):
        self.client.close()



from tests.integration import base
from wildfly.api.deployment import DEFAULT_SERVER_GROUP


class HostTest(base.BaseTestCase):
    def test_list_hosts(self):
        hosts = self.client.hosts()
        self.assertIn('master', hosts)

    def test_list_hosts_in_group(self):
        hosts = self.client.hosts(server_group=DEFAULT_SERVER_GROUP)
        self.assertIn('master', hosts)



import responses
import json
from wildfly.client import Client
from .. import base

try:
    from unittest import mock
except ImportError:
    import mock


class ClientTest(base.BaseTestCase):

	def test_client_init_default(self):
		client = Client("hostname")
		self.assertEqual(client.username, 'admin')
		self.assertEqual(client.password, 'admin')
		self.assertEqual(client.host, "hostname")
		self.assertEqual(client.port, "9990")
		self.assertEqual(client.timeout, 5000)
		self.assertEqual(client.endpoint, 'http://hostname:9990/management')

	@mock.patch.object(Client, 'execute')
	def test_Client(self, mock_execute):
		mock_execute.return_value = 'executed'
		client = Client('hostname')
		client.execute('add')

		self.assertEqual(mock_execute.return_value, client.execute('add'))
		mock_execute.assert_called_with('add')

		client.execute('add', {})
		mock_execute.assert_called_with('add', {})

		client.execute("add", {}, [])
		mock_execute.assert_called_with('add', {}, [])

	@responses.activate
	def test_post_200(self):
		client = Client('host')

		responses.add(responses.POST, client.endpoint, json={}, status=200)
		resp = client._post(json.dumps({}))

		assert resp.status_code == 200
		assert responses.calls[0].request.url == client.endpoint
		assert responses.calls[0].response.text == ''

	@responses.activate
	def test_post_500(self):
		client = Client('host')
		responses.add(responses.POST, client.endpoint, json={}, status=500)

		resp = client._post('hi')
		assert resp.status_code == 500

	@mock.patch('wildfly.client.Client._post')
	def test_execute_operation(self, mock_post):
		client = Client("hostname")

		client.execute("add")
		mock_post.assert_called_with({'operation': 'add', 'address': []})

		client.execute("add", address=['localhost'])
		mock_post.assert_called_with({'operation': 'add', 'address': ['localhost']})

		client.execute("add", parameters={"hello": "world"})
		mock_post.assert_called_with({'operation': 'add', 'hello': 'world', 'address': []})

	@mock.patch('wildfly.client.Client.execute')
	def test_add_address(self, mock_execute):
		client = Client("hostname")

		client.add('address')
		mock_execute.assert_called_with('add', None, 'address')

	@mock.patch('wildfly.client.Client.execute')
	def test_add_address_and_parameters(self, mock_execute):
		client = Client("hostname")
		mock_execute.return_value = 'added'
		response = client.add('address', 'parameters')
		mock_execute.assert_called_with('add', 'parameters', 'address')
		self.assertEqual(response, mock_execute.return_value)

	@mock.patch('wildfly.client.Client.remove')
	def test_remove(self, mock_remove):
		mock_remove.return_value = "removed"

		client = Client('hostname')
		client.remove('address')

		self.assertEqual(mock_remove.return_value, client.remove('address'))
		mock_remove.assert_called_with('address')

	@mock.patch('wildfly.client.Client.execute')
	def test_remove_return_value(self, mock_execute):
		mock_execute.return_value = None
		client = Client("hostname")

		self.assertIsNone(client.remove('address'))

	@mock.patch('wildfly.client.Client.read_resource')
	def test_read_resource(self, mock_read_resource):
		mock_read_resource.return_value = 'resource'

		client = Client('hostname')

		self.assertEqual(mock_read_resource.return_value, client.read_resource('address'))
		mock_read_resource.assert_called_with('address')

	@mock.patch('wildfly.client.Client.execute')
	def test_read_resource_return_value(self, mock_execute):
		mock_execute.return_value = None
		client = Client("hostname")

		self.assertIsNone(client.read_resource('address'))

	@mock.patch('wildfly.client.Client.read_attribute')
	def test_read_attribute(self, mock_read_attribute):
		mock_read_attribute.return_value = 'attribute'
		client = Client('hostname')

		self.assertEqual(mock_read_attribute.return_value, client.read_attribute('attribute_name'))
		mock_read_attribute.assert_called_with('attribute_name')

	@mock.patch('wildfly.client.Client.execute')
	def test_read_attribute_return_none(self, mock_execute):
		mock_execute.return_value = None
		client = Client('hostname')

		self.assertIsNone(client.read_attribute('attribute_name'))

	@mock.patch('wildfly.client.Client.write_attribute')
	def test_write_attribute(self, mock_write_attribute):
		mock_write_attribute.return_value = 'attribute_written'
		client = Client('hostname')

		self.assertEqual(mock_write_attribute.return_value, client.write_attribute('name', 'value'))
		mock_write_attribute.assert_called_with('name', 'value')

	@mock.patch('wildfly.client.Client.execute')
	def test_write_attribute_return_value(self, mock_execute):
		mock_execute.return_value = None

		client = Client('hostname')

		self.assertIsNone(client.write_attribute('name', 'value'))

	@mock.patch('wildfly.client.Client.unset_attribute')
	def test_unset_attribute(self, mock_unset_attribute):
		mock_unset_attribute.return_value = 'unset_attribute'
		client = Client('hostname')

		self.assertEqual(mock_unset_attribute.return_value, client.unset_attribute('name'))
		mock_unset_attribute.assert_called_with('name')

	@mock.patch('wildfly.client.Client.execute')
	def test_unset_attribute_return_none(self, mock_execute):
		mock_execute.return_value = None

		client = Client('hostname')

		self.assertIsNone(client.unset_attribute('name'))

	@mock.patch('wildfly.client.Client.read_children_names')
	def test_read_child_names(self, mock_read_children_names):
		mock_read_children_names.return_value = 'children_name'
		client = Client('hostname')

		self.assertEqual(mock_read_children_names.return_value, client.read_children_names('child_type'))
		mock_read_children_names.assert_called_with('child_type')

	@mock.patch('wildfly.client.Client.execute')
	def test_read_child_names_return_none(self, mock_execute):
		mock_execute.return_value = None

		client = Client('hostname')

		self.assertIsNone(client.read_children_names('child_type'))

	@mock.patch('wildfly.client.Client.read_children_resources')
	def test_read_children_resources(self, mock_read_children_resources):
		mock_read_children_resources.return_value = 'children_resources'
		client = Client('hostname')

		self.assertEqual(mock_read_children_resources.return_value, client.read_children_resources('child_type'))
		mock_read_children_resources.assert_called_with('child_type')

	@mock.patch('wildfly.client.Client.execute')
	def test_read_children_resources_return_none(self, mock_execute):
		mock_execute.return_value = None

		client = Client('hostname')

		self.assertIsNone(client.read_children_resources('child_type'))

	@mock.patch('wildfly.client.Client.read_operation_names')
	def test_read_operation_names(self, mock_read_operation_names):
		mock_read_operation_names.return_value = 'operation_name'
		client = Client("hostname")

		self.assertEqual(mock_read_operation_names.return_value, client.read_operation_names())
		mock_read_operation_names.assert_called_with()

	@mock.patch('wildfly.client.Client.execute')
	def test_read_operation_names_return_none(self, mock_execute):
		mock_execute.return_value = None

		client = Client('hostname')

		self.assertIsNone(client.read_operation_names())

	@mock.patch('wildfly.client.Client.read_operation_description')
	def test_read_operation_description(self, mock_read_operation_description):
		mock_read_operation_description.return_value = 'operation_description'
		client = Client("hostname")

		self.assertEqual(mock_read_operation_description.return_value, client.read_operation_description('operation_name'))
		mock_read_operation_description.assert_called_with('operation_name')

	@mock.patch('wildfly.client.Client.execute')
	def test_read_operation_description_return_none(self, mock_execute):
		mock_execute.return_value = None

		client = Client('hostname')

		self.assertIsNone(client.read_operation_description('operation_name'))

	@mock.patch('wildfly.client.Client.read_children_types')
	def test_read_children_types(self, mock_read_children_types):
		mock_read_children_types.return_value = 'description'
		client = Client("hostname")

		self.assertEqual(mock_read_children_types.return_value, client.read_children_types())
		mock_read_children_types.assert_called_with()

	@mock.patch('wildfly.client.Client.execute')
	def test_read_children_types_return_none(self, mock_execute):
		mock_execute.return_value = None

		client = Client('hostname')

		self.assertIsNone(client.read_children_types())

	@mock.patch('wildfly.client.Client.read_attribute')
	def test_version(self, mock_read_attribute):
		mock_read_attribute.return_value = 'version'
		client = Client('hostname')

		self.assertEqual('version', client.version())


import socket

from tests.integration import base
from wildfly.api.deployment import DEFAULT_SERVER_GROUP


class StartServersTest(base.BaseTestCase):

    def test_start_servers(self):
        self.client.start_servers(blocking=True)
        servers = self.client.servers()
        for key in servers:
            self.assertEqual(servers[key]['status'], 'running')

    def test_start_server_group(self):
        self.client.start_servers(
            server_group=DEFAULT_SERVER_GROUP,
            blocking=True)
        servers = self.client.servers(server_group=DEFAULT_SERVER_GROUP)
        for key in servers:
            self.assertEqual(servers[key]['status'], 'running')


class StopServersTest(base.BaseTestCase):

    def test_stop_servers(self):
        self.client.stop_servers(blocking=True)
        servers = self.client.servers()
        for key in servers:
            self.assertEqual(servers[key]['status'], 'STOPPED')
        self.client.start_servers(blocking=True)

    def test_stop_server_group(self):
        self.client.stop_servers(
            server_group=DEFAULT_SERVER_GROUP,
            blocking=True)
        servers = self.client.servers(server_group=DEFAULT_SERVER_GROUP)
        for key in servers:
            self.assertEqual(servers[key]['status'], 'STOPPED')
        self.client.start_servers(
            server_group=DEFAULT_SERVER_GROUP,
            blocking=True)


class ReloadServersTest(base.BaseTestCase):

    def test_reload_servers(self):
        self.client.reload_servers(blocking=True)
        servers = self.client.servers()
        for key in servers:
            self.assertEqual(servers[key]['status'], 'running')

    def test_reload_server_group(self):
        self.client.reload_servers(
            server_group=DEFAULT_SERVER_GROUP,
            blocking=True)
        servers = self.client.servers(server_group=DEFAULT_SERVER_GROUP)
        for key in servers:
            self.assertEqual(servers[key]['status'], 'running')


class RestartServersTest(base.BaseTestCase):

    def test_restart_servers(self):
        self.client.restart_servers(blocking=True)
        servers = self.client.servers()
        for key in servers:
            self.assertEqual(servers[key]['status'], 'running')

    def test_restart_server_group(self):
        self.client.restart_servers(
            server_group=DEFAULT_SERVER_GROUP, blocking=True)
        servers = self.client.servers(server_group=DEFAULT_SERVER_GROUP)
        for key in servers:
            self.assertEqual(servers[key]['status'], 'running')



import logging


logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)


class HostApiMixin(object):

    def hosts(self, server_group=None):
        """ Returns list of hosts. """

        # TODO need to implement filter for only hosts in specified
        #  server_group
        hosts = self.read_children_names('host')
        logger.debug('HOSTS: {}'.format(hosts))
        return hosts

    def _get_hostname_map(self):
        """
        Returns a dictionary that has a mapping of the physical machine
        qualified_hostname to a WF host and WF server.

        The dictionary has the following sample structure
            { 'wf-hos-2t': 'wf-srvr-1': { 'srvr-grp-1': "qualified-host-name',
                                          'srvr-grp-2': "qualified-host-name'},
                           'wf-srvr-2': { 'srvr-grp-3': "qualified-host-name'},
              'wf-host-2': 'wf-srvr-3': { 'srvr-grp-4': "qualified-host-name'},
                           'wf-srvr-4': { 'srvr-grp-5': "qualified-host-name',
                                          'srvr-grp-6': "qualified-host-name',
                                          'srvr-grp-7': "qualified-host-name'}
            }
        :return a dictionary that has a mapping of the WF host, server and
                server group to a physically qualified domain name
        """
        logger.debug("Generating Hostname map")
        host_map = {}
        for host in self.hosts():
            logger.debug("host is {host}".format(host=host))
            if host not in host_map:
                # Ensure we have the host key is in the host_map
                host_map[host] = {}
            # Get the server name
            server_name = self.get_server_name(host)

            logger.debug("server name is {server}".format(server=server_name))
            if server_name not in host_map[host]:
                # Ensure the server name is a key under the host
                host_map[host][server_name] = {}

            if server_name:
                try:
                    # Not all WildFly server names will have a server group,
                    # so we need to wrap it with a try catch.
                    sg = self.get_server_group(host, server_name)
                    qualified_hostname = self.get_hostname(host, server_name)

                    # Set the Server group and qualified_hostname
                    host_map[host][server_name][sg] = qualified_hostname
                    logger.debug("WildFly host {wfhost} and server {server} has "
                                 "a server group {sg} and qualified_hostname of "
                                 "{host}".format(wfhost=host,
                                                 server=server_name,
                                                 sg=sg,
                                                 host=qualified_hostname,))
                except (SystemError, KeyError) as e:
                    # if an error occurs, the WF server may not have a
                    # server-group as it may be the data container. Just log
                    # the error and "pass"
                    logger.error("(NOT A BUG) A failure occurred: "
                                 "{err}".format(err=e))

        logger.debug("done generating hostname map.  The Hostname map is {map}".format(map=host_map))
        return host_map



import os
import errno
import logging
import requests
from .. import util


DEFAULT_CONTENT_HOST = 'nexus'
DEFAULT_CONTENT_HOST_ENDPOINT = None
DEFAULT_CONTENT_HOST_PORT = '8081'
DEFAULT_SERVER_GROUP = 'main-server-group'
DEFAULT_ARTIFACT_TYPE = 'jar'

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)


class DeploymentApiMixin(object):

    def deployments(self, server_group=None, host=None):
        """ Returns information about deployments. """

        # status['RUNNING', 'STOPPED', 'FAILED']
        deployments = self.read_children_resources('deployment')
        for key in deployments:
            deployments[key]['server-groups'] = []
            deployments[key]['enabled'] = False
            deployments[key]['status'] = 'STOPPED'
            deployments[key]['hosts'] = []

        server_groups = self.server_groups()
        for _server_group in server_groups:
            deployments_in_group = self.read_children_resources(
                'deployment', [{'server-group': _server_group}], runtime=True)
            for key in deployments_in_group:
                deployments[key]['enabled'] = deployments_in_group[
                    key]['enabled']
                if 'server-groups' in deployments[key]:
                    deployments[key]['server-groups'].append(_server_group)
                else:
                    deployments[key]['server-groups'] = [_server_group]

        servers = self.servers()

        for server in servers:
            if servers[server]['status'] != 'STOPPED':
                deployments_on_server = self.read_children_resources(
                    'deployment',
                    [{'host': servers[server]['host']}, {'server': server}],
                    runtime=True)
                logger.debug(
                    'DEPLOYMENTS_ON_SERVER({}): {}'.format(
                        server, deployments_on_server))

                for key in deployments_on_server:
                    if deployments_on_server[key]['status'] == 'OK':
                        deployments[key]['status'] = 'RUNNING'
                    deployments[key]['hosts'].append(servers[server]['host'])

        if server_group:
            for key in deployments.keys():
                if server_group not in deployments[key]['server-groups']:
                    del deployments[key]

        if host:
            for key in deployments.keys():
                if host not in deployments[key]['hosts']:
                    del deployments[key]

        return deployments

    def is_deployment_in_repository(self, name):
        """ Check if deployment exists in content repository. """
        return name in self.deployments()

    def is_deployment_enabled(self, name):
        """ Check if deployment is enabled. """
        return self.deployments()[name]['enabled']

    def deployment_status(self, name):
        """ Get deployment status. """
        return self.deployments()[name]['status']

    def pull(
            self,
            groupId,
            artifactId,
            version,
            type='war',
            server_groups=DEFAULT_SERVER_GROUP,
            path=None,
            content_host=DEFAULT_CONTENT_HOST,
            content_host_ep=DEFAULT_CONTENT_HOST_ENDPOINT,
            content_host_port=DEFAULT_CONTENT_HOST_PORT):
        """ Pull artifact from artifact repository into wildfly content
        repository. """

        self.deploy(groupId, artifactId, version, type, server_groups,
                    path, enabled=False,
                    content_host=content_host, content_host_ep=content_host_ep,
                    content_host_port=content_host_port)

    def deploy(
            self,
            groupId,
            artifactId,
            version,
            type=DEFAULT_ARTIFACT_TYPE,
            server_groups=DEFAULT_SERVER_GROUP,
            path=None,
            enabled=True,
            force=True,
            content_host=DEFAULT_CONTENT_HOST,
            content_host_ep=DEFAULT_CONTENT_HOST_ENDPOINT,
            content_host_port=DEFAULT_CONTENT_HOST_PORT,
            scheme="http"):

        """ Deploy artifact to WildFly. """
        if path is None:
            if content_host_ep == 'nexus':
                if 'SNAPSHOT' not in version:
                    BASE_URL = '{}://{}:{}/{}/service/local/repo_groups' \
                           '/public/content'.format(scheme,
                                                    content_host,
                                                    content_host_port,
                                                    content_host_ep)

                    url = '{0}/{1}/{2}/{3}/{2}-{3}.{4}'.format(
                        BASE_URL,
                        groupId.replace('.', '/'),
                        artifactId,
                        version,
                        type
                    )
                else:
                    BASE_URL = '{}://{}:{}/{}/service/local/artifact' \
                           '/maven/content?r=public'.format(scheme,
                                                            content_host,
                                                            content_host_port,
                                                            content_host_ep)
                    url = '{0}&g={1}&a={2}&v={3}&p={4}'.format(
                        BASE_URL,
                        groupId.replace('.', '/'),
                        artifactId,
                        version,
                        type
                    )

            elif content_host_ep == 'maven2':
                BASE_URL = '{}://{}:{}/{}'.format(scheme,
                                                  content_host,
                                                  content_host_port,
                                                  content_host_ep)
                url = '{0}/{1}/{2}/{3}/{2}-{3}.{4}'.format(
                    BASE_URL,
                    groupId.replace('.', '/'),
                    artifactId,
                    version,
                    type)
            elif content_host_ep is None:
                BASE_URL = '{}://{}:{}/service/local/repositories/releases/content'.format(scheme,
                                                                                           content_host,
                                                                                           content_host_port)
                url = '{0}/{1}/{2}/{3}/{2}-{3}.{4}'.format(
                    BASE_URL,
                    groupId.replace('.', '/'),
                    artifactId,
                    version,
                    type)
            else:
                # Not supported
                raise Exception("Content host type {} not supported"
                                .format(content_host_ep))

            # check if url exists
            response = requests.head(url)
            if response.status_code is not 200:
                response.raise_for_status()

            # upload artifact from url to content repository
            response = self.execute('upload-deployment-url', {'url': url})
        else:

            # check if file exists
            if not os.path.isfile(path):
                raise IOError(errno.ENOENT, os.strerror(errno.ENOENT), path)

            # upload artifact from local file path to content repository
            files = {'file': open(path, 'rb')}
            response = requests.post(
                self.endpoint + '/add-content',
                files=files,
                auth=requests.auth.HTTPDigestAuth(
                    self.username,
                    self.password))
            logger.debug(
                'Response Status Code: {}: {}'.format(
                    response.status_code,
                    response.reason))
            logger.debug('Response: {}'.format(response.json()))

        # TODO support new deploy and redeploy
        # TODO if deploy fails then rollback to previous
        # if force:
        # if isDeploymentInRepository("{}.{}".format(artifactId, type)):
        # replaceDeployment(ctx, f, deploymentUrl, name, runtimeName, disabled)
        # return

        # add artifact to content repository
        byte_value = response.json()['result']['BYTES_VALUE']

        # https://github.com/cenx-cf/wildfly-py/issues/5
        if type == 'war':
            runtime_name = artifactId + '-' + version + '.' + type
            name = artifactId + '.' + type
        elif type == 'jar':
            try:
                runtime_name = artifactId.split('-')[-2] \
                               + '-resources' + '.' + type
            except IndexError:
                runtime_name = artifactId + '-' + version + '.' + type
            name = runtime_name
            
        request = {"content": [{"hash": {"BYTES_VALUE": byte_value}}],
                   "address": [{"deployment": "{}".format(name)}],
                   "operation": "add",
                   "runtime-name": runtime_name}
                
        response = self._post(request)

        # add artifact to server-group(s)
        address = [{'server-group': server_groups},
                   {'deployment': '{}'.format(name)}]
        response = self.add(address, {'enabled': enabled})

    def disable(self, name, server_groups=DEFAULT_SERVER_GROUP):
        """ Disable artifact in given server_groups, but keep artifact in
        assigned server_groups. """
        address = [{'server-group': server_groups},
                   {'deployment': name}]

        response = self.execute('undeploy', address=address)
        if not util.is_success(response):
            logging.debug("Failed to disable artifact: {}".format(name))

    def enable(self, name, server_groups=DEFAULT_SERVER_GROUP):
        """ Enable artifact in given server_groups. """
        address = [{'server-group': server_groups},
                   {'deployment': name}]

        response = self.execute('deploy', address=address)
        if not util.is_success(response):
            logging.debug("Failed to enable artifact: {}".format(name))

    def undeploy(self, name, server_groups=DEFAULT_SERVER_GROUP):
        """ Undeploy artifact from WildFly. """

        # remove deployment from server-group(s)
        address = [{'server-group': server_groups},
                   {'deployment': name}]
        response = self.remove(address)
        if util.is_success(response) is True:
            # remove deployment from content repository
            address = [{"deployment": name}]
            response = self.remove(address)
        return response



from wildfly.client import Client
from .. import base

try:
    from unittest import mock
except ImportError:
    import mock


class HostApiMixinTest(base.BaseTestCase):

	@mock.patch('wildfly.client.Client.read_children_names')
	def test_hosts(self, mock_read_children_names):
		mock_read_children_names.return_value = 'hosts'
		client = Client('hostname')
		self.assertEqual('hosts',  client.hosts())
		mock_read_children_names.assert_called_with('host')






    # python binding for wildlfy management http/json api
import logging
import json
import requests

from . import util
from . import api

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)

# Define some
KEY_OUTCOME = "outcome"
KEY_RESULT = "result"


class Client(requests.Session,
             api.HostApiMixin,
             api.ServerApiMixin,
             api.DeploymentApiMixin):

    DEFAULT_MANAGEMENT_PORT = '9990'
    DEFAULT_MANAGEMENT_USER = 'admin'
    DEFAULT_MANAGEMENT_PWD = 'admin'
    DEFAULT_TIMEOUT = 5000

    def __init__(
            self,
            host,
            port=DEFAULT_MANAGEMENT_PORT,
            username=DEFAULT_MANAGEMENT_USER,
            password=DEFAULT_MANAGEMENT_PWD,
            timeout=DEFAULT_TIMEOUT):

        super(Client, self).__init__()
        self.username = username
        self.password = password
        self.host = host
        self.port = port
        self.timeout = timeout
        self.endpoint = 'http://{}:{}/management'.format(self.host, self.port)

    def _post(self, request):

        logger.debug('Request: {}'.format(request))
        headers = {'content-type': 'application/json'}
        response = requests.post(
            self.endpoint,
            headers=headers,
            auth=requests.auth.HTTPDigestAuth(
                self.username,
                self.password),
            data=json.dumps(request))
        if response.status_code in [200, 204]:
            logger.debug(
                'Response Status Code: {}: {}'.format(
                    response.status_code,
                    response.reason))
        elif response.status_code == 500:
            logger.debug(
                'Response Status Code: {}: {}'.format(
                    response.status_code,
                    response.json))
        else:
            response.raise_for_status()
        logger.debug('Response: {}'.format(response.json))
        return response

    def execute(self, operation, parameters={}, address=[]):
        """ Execute operation on resource. """

        request = {'address': address, 'operation': operation}
        request.update(parameters)
        return self._post(request)

    def add(self, address, parameters=None):
        """ Creates a new management resource. """

        response = self.execute('add', parameters, address)
        return response

    def remove(self, address):
        """ Removes an existing resource. """

        response = self.execute('remove', address=address)
        return response

    def read_resource(
            self,
            address=[],
            recursive=False,
            recursive_depth=10,
            runtime=False,
            include_defaults=True,
            attributes_only=False):
        """ Reads a resource's attribute values along with either
            basic or complete information about any child resources. """

        response = self.execute('read-resource',
                                {'recursive': recursive,
                                 'recursive_depth': recursive_depth,
                                 'runtime': runtime,
                                 'include_defaults': include_defaults,
                                 'attributes_only': attributes_only},
                                address)
        return response

    def read_attribute(self, name, address=[], include_defaults=True):
        """ Read attribute of resource. """

        response = self.execute('read-attribute',
                                {'name': name,
                                 'include-defaults': include_defaults},
                                address)
        return response.json()['result'] if util.is_success(response) else None

    def write_attribute(self, name, value, address=[]):
        """ Write value of attribute of resource. """

        response = self.execute('write-attribute',
                                {'name': name, 'value': value},
                                address)
        return response

    def unset_attribute(self, name, address=[]):
        """ Sets the value of an individual attribute to the undefined
        value. """

        response = self.execute('unset-attribute',
                                {'name': name},
                                address)
        return response.json()['result'] if util.is_success(response) else None

    def read_children_names(self, child_type, address=[]):
        """ Returns a list of the names of all child resources of a
        given type. """

        response = self.execute('read-children-names',
                                {'child-type': child_type},
                                address)
        return response.json()['result'] if util.is_success(response) else None

    def read_children_resources(self, child_type, address=[], runtime=False):
        """
        Returns a list of the resources of all child resources of a
        given type.
        """

        response = self.execute('read-children-resources',
                                {'child-type': child_type,
                                 'include-runtime': runtime},
                                address)
        return response.json()['result'] if util.is_success(response) else None

    def read_operation_names(self, address=[]):
        """ Returns a list of the names of all the operations the resource
        supports. """

        response = self.execute('read-operation-names',
                                address)
        return response.json()['result'] if util.is_success(response) else None

    def read_operation_description(self, name, address=[]):
        """ Returns the description of an operation, along with details of
        its parameter types and its return value.
        """

        response = self.execute('read-operation-description',
                                {'name': name},
                                address)
        return response.json()['result'] if util.is_success(response) else None

    def read_children_types(self, address=[]):
        """ Returns a list of the types of child resources the resource
        supports. """

        response = self.execute('read-children-types',
                                address)
        return response.json()['result'] if util.is_success(response) else None

    def version(self):
        """ Prints version of WildFly. """

        result = self.read_attribute('release-version')
        return result

    def get_raw_server_groups_info(self):
        """
        Run the equivalent WildFly CLI command   /server-group=*:read-resource
        :return a dictionary representing the response
        """

        address = [{'server-group': '*'}]
        resp = self.read_resource(address)
        return resp.json()

    def get_server_name(self, host):
        """
        Return a single
        Runs the equivalent WF CLI command: ls /host=<host>

        :param host  A Wildfly host as specified by the `ls /hosts` CLI command
        :return A string representing a WF Server name for the given WF host.
        """
        logger.debug("Retrieving server name for host '{host}'."
                    .format(host=host))
        address = [{'host': host}, {'server': '*'}]
        result = self.read_resource(address).json()

        logger.debug("Retrieved server name for wf host {host}.  Response is "
                    "{resp}".format(host=host, resp=result))
        status = result.get(KEY_OUTCOME, None)
        if status != 'success':
            raise Exception("Something bad happened when trying th get servers "
                            "of WF host {host}".format(host=host))

        logger.debug("Command is successful, processing response now...")
        server = None
        if KEY_RESULT in result and result[KEY_RESULT]:
            for list_item in result[KEY_RESULT]:
                address = list_item.get('address', [])
                for item in address:
                    if 'server' in item:
                        server = item['server']
        return server

    def get_server_group(self, host, server_name):
        """
        Gets the server group associated with a WF host and WF server.  Runs
        the equivalent WF CLI command:
             /host=<host>/server=<server>:read_attribute(name=server-group,include-defaults=true)
        :param host  The WildFly host
        :param server_name  The WildFly server
        :return The value of the server-group
        """
        logger.debug("Retrieving server-group for host '{host}' and server "
                     "'{server}'...".format(host=host,
                                            server=server_name))
        address = [{'host': host}, {'server': server_name}]
        val = self.read_attribute('server-group', address, include_defaults=True)
        logger.debug("WildFly host {host} and server {server} has a server-group"
                     " of {sg}".format(host=host,
                                       server=server_name,
                                       sg=val))
        return val

    def get_hostname(self, host, svr_name):
        """
        Returns the fully qualified host nanme associated with a given WF host
        and WF server.  Runs the equivalent WF CLI command:
            /host=<host>/server=<server>/core-service=server-environment:read_attribute(name=qualified-host-name,include-defaults=true)
        :param host  The WildFly host
        :param svr_name  The WildFly server
        :return The associated qualified host name
        """
        logger.debug("Retrieving qualified hostname for host '{host}' and "
                     "server '{server}'...".format(host=host,
                                                   server=svr_name))
        address = [{'host': host},
                   {'server': svr_name},
                   {'core-service': 'server-environment'}]
        val = self.read_attribute('qualified-host-name',
                                  address,
                                  include_defaults=True)
        logger.debug("WildFly host {wfhost} and server {server} has a hostname "
                     "of {host}".format(wfhost=host,
                                        server=svr_name,
                                        host=val))
        return val

    def get_server_group_host(self, server_group):
        """
        Returns the hostname of a server group
        :param server_group  The server group
        :return hostname associated with a server group
        """
        if "_hostname_map" not in self.__dict__ or \
                        self._hostname_map is None or \
                        len(self._hostname_map) == 0:
            # Initialize the hostname map if self._hostname_map is not defined
            # in the class object, or is None, or is an empty dictionary
            self._hostname_map = self._get_hostname_map()

        logger.info("Getting server group {sg} hostname.".format(
            sg=server_group))
        hostnames = [hostname
                     for wf_host in self._hostname_map
                     for wf_server in self._hostname_map[wf_host]
                     for sg, hostname in self._hostname_map[wf_host][wf_server].items()
                     if server_group == sg]
        logger.info("Server Group {sg} is running on hostnames "
                    "{hostnames}".format(sg=server_group,
                                         hostnames=", ".join(hostnames)))
        return hostnames

    def get_application_hostnames(self, application, ext=None):
        """
        Get the hostname where an application is deployed
        :param application The application
        :param ext The application extension  e.g. war, jar
        :return A list of host names where the application is running
        """

        # Let's find the server groups that the apps is deployed against,
        # because an app can be deployed to more than one server group
        if ext:
            application = ".".join([application, ext])

        logger.info("Getting hostnames where application {app} is running."
                    .format(app=application))
        # Get a list of deployed all
        sg_info = self.get_deployed_apps()
        server_groups = [sg
                         for sg in sg_info
                         for app in sg_info[sg]
                         if application in app]
        logger.info("Application {app} is deployed in server-groups: "
                    "{sg}".format(app=application,
                                  sg=", ".join(server_groups)))
        hosts_list = set([host
                          for sg in server_groups
                          for host in self.get_server_group_host(sg)])
        logger.info("Application {app} is deployed on host names: "
                    "{hosts}".format(app=application,
                                     hosts=", ".join(hosts_list)))

        return hosts_list



#!/usr/bin/env python

# Setuptools install description file.

from setuptools import setup, find_packages

with open('README.md') as readme:
    long_description = readme.read()

with open('requirements.txt') as f:
    requirements = [line.strip() for line in f.readlines()]

setup(
    name='wildfly-py',
    version='0.0.4',
    description='WildFly Management Python API',
    long_description=long_description,
    zip_safe=True,
    packages=find_packages(),
    install_requires=requirements,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Operating System :: Unix',
        'Programming Language :: Python',
        'Topic :: Utilities'],
    entry_points={
        'console_scripts': ['wildfly-py = wildfly.__main__:main'],
        'setuptools.installation': ['eggsecutable = wildfly.__main__:main'],
    },
    author='CENX',
    author_email='devops@cenx.com',
    license='Apache License 2.0',
    keywords='cenx wildfly',
    url='https://github.org/cenx-cf/wildfly-py')



import logging


logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)


class ServerApiMixin(object):

    def servers(self, server_group=None, host=None):
        hosts = self.hosts()
        servers_merged = {}
        for host in hosts:
            address = [{'host': host}]
            servers_config = self.read_children_resources(
                'server-config', address, True)
            logger.debug('SERVERS_CONFIG: {}'.format(servers_config))
            servers = self.read_children_resources('server', address, True)
            logger.debug('SERVERS: {}'.format(servers))
            if servers is not None:
              for key, value in servers.items():
                  if value['server-state'] != 'STOPPED':
                      address = [{'host': host}, {'server': key}, {
                          'core-service': 'platform-mbean'}, {'type': 'runtime'}]
                      uptime = self.read_attribute(
                          address=address, name='uptime')
                  else:
                      uptime = None
                  if server_group is None or servers_config[
                          key]['group'] == server_group:
                      # STARTING, RUNNING, STOPPED or RESTART_REQUIRED
                      servers_merged[key] = {
                          'group': servers_config[key]['group'],
                          'host': host,
                          'status': value['server-state'],
                          'uptime': uptime}
        logger.debug('SERVERS_MERGED: {}'.format(servers_merged))
        return servers_merged

    def server_groups(self):
        server_groups = self.read_children_resources('server-group')
        return server_groups

    def _server_operation(self, operation, server_group=None, blocking=False):
        if server_group:
            address = [{'server-group': server_group}]
        else:
            address = []
        return self.execute(operation, {'blocking': blocking}, address)

    def start_servers(self, server_group=None, blocking=False):
        """ Starts all configured servers in domain or
        server_group that are not currently running. """
        return self._server_operation('start-servers', server_group, blocking)

    def stop_servers(self, server_group=None, blocking=False):
        """ Stops all servers currently running in the domain or
        server_group. """
        return self._server_operation('stop-servers', server_group, blocking)

    def reload_servers(self, server_group=None, blocking=False):
        """ Reloads all servers currently running in the domain. """
        return self._server_operation('reload-servers', server_group, blocking)

    def restart_servers(self, server_group=None, blocking=False):
        """ Restart all servers currently running in the domain or
        server_group. """
        return self._server_operation(
            'restart-servers', server_group, blocking)

    def read_log_file(self, host=None):
        """ Reads the contents of a log file. The file must be in the
        jboss.server.log.dir and must be defined as a file-handler,
        periodic-rotating-file-handler or size-rotating-file-handler. """

        if host is None:
            hosts = self.hosts()
            host = hosts[0]

        address = [{'host': host},
                   {'server': '{}-0'.format(host)},
                   {'subsystem': 'logging'}]
        parameters = {'name': 'server.log', 'tail': 'true', 'lines': '100'}
        response = self.execute('read-log-file', parameters, address)
        return response.json()['result']



import logging
from .client import Client  # flake8: noqa

# setup log stream handler
ch = logging.StreamHandler()
ch.setLevel(logging.DEBUG)
formatter = logging.Formatter('%(asctime)s | %(levelname)7s | %(name)s | line:%(lineno)4s | %(message)s')
ch.setFormatter(formatter)

# setup package logger
logger = logging.getLogger(__name__)
#logger.setLevel(logging.INFO)
logger.addHandler(ch)




import responses
import requests
import urllib
from wildfly.client import Client
from .. import base

try:
    from unittest import mock
except ImportError:
    import mock


class DeploymentApiMixinTest(base.BaseTestCase):

	@mock.patch('wildfly.client.Client.read_children_resources')
	@mock.patch('wildfly.api.server.ServerApiMixin.server_groups')
	@mock.patch('wildfly.api.server.ServerApiMixin.servers')
	def test_deployments(self, mock_servers, mock_server_groups, mock_read_children_resources):
		def my_side_effect(*args, **kwargs):
			if args[0] == 'deployment' and len(args) == 1:
				return {'application': {'content': [{'hash': {'BYTES_VALUE': 'hash'}}],
										'runtime-name': 'runtime_name',
										'name': 'name'}}
			else:
				return {u'application': {u'status': u'OK',
										 u'subsystem': {u'undertow': None},
										 u'name': u'name',
										 u'enabled': True,
										 u'persistent': True,
										 u'content': [{u'hash': {u'BYTES_VALUE': u'hash'}}],
										 u'runtime-name': u'runtime_name',
										 u'subdeployment': None}}

		mock_servers.return_value = {'server': {'status': 'running', 'host': 'hostname', 'group': 'group', 'uptime': 123}}
		mock_server_groups.return_value  = {'server_group': {u'profile': u'full-ha',
		                                                     u'socket-binding-group': u'full-ha-sockets',
		                                                     u'deployment-overlay': None,
		                                                     u'socket-binding-port-offset': 0,
		                                                     u'jvm': None,
		                                                     u'deployment': {u'application': None},
		                                                     u'system-property': None,
		                                                     u'management-subsystem-endpoint': False}}

		mock_read_children_resources.side_effect = my_side_effect

		client = Client('hostname')

		self.assertEqual('RUNNING', client.deployments('server_group', 'hostname')['application']['status'])

		self.assertDictEqual({}, client.deployments(server_group='server_group_not_exist', host=None))


	@mock.patch('wildfly.api.deployment.DeploymentApiMixin.deployments')
	def test_is_deployment_in_repository(self, mock_deployments):
		mock_deployments.return_value = {'name': 'artifact'}
		client = Client('hostname')

		self.assertTrue(client.is_deployment_in_repository('name'))

	@mock.patch('wildfly.api.deployment.DeploymentApiMixin.deployments')
	def test_is_deployment_enabled(self, mock_deployments):
		mock_deployments.return_value = {'name': {'enabled': True}}
		client = Client('hostname')

		self.assertTrue(client.is_deployment_enabled('name'))

	@mock.patch('wildfly.api.deployment.DeploymentApiMixin.deployments')
	def test_deployment_status(self, mock_deployments):
		mock_deployments.return_value = {'name': {'status': 'running'}}
		client = Client('hostname')

		self.assertEqual('running', client.deployment_status('name'))

	@mock.patch('wildfly.api.deployment.DeploymentApiMixin.deploy')
	def test_pull(self, mock_deploy):
		mock_deploy.return_value = True
		client = Client('hostname')
		client.pull('groupId', 'artifactId', 'version', 'war', 'A', None, 'content_host', 'endpoint', 'port')

		mock_deploy.assert_called_with('groupId', 'artifactId', 'version', 'war', 'A', None, content_host='content_host', content_host_ep='endpoint', content_host_port='port', enabled=False)


	@mock.patch('__builtin__.open', spec=open)
	@mock.patch('wildfly.api.deployment.os.path.isfile')
	@responses.activate
	def test_deploy(self, mock_isfile, mock_open):
		mock_isfile.return_value = True
		mock_open.return_value = 'file_content'


		# path is None, end_point is nexus and snapshot
		groupId = 'group_id'
		artifactId = 'artifact_id'
		type = 'war'
		server_groups = 'server_groups'
		path = None
		enabled = True
		force = True
		content_host = 'content_host'
		content_host_ep = 'nexus'
		content_host_port = 8081
		scheme='http'


		client = Client('hostname')


		# path is None, end_point is nexus and non-snapshot
		version = 'version'
		BASE_URL = '{}://{}:{}/{}/service/local/repo_groups' \
		           '/public/content'.format(scheme,
		                                    content_host,
		                                    content_host_port,
		                                    content_host_ep)

		url = '{0}/{1}/{2}/{3}/{2}-{3}.{4}'.format(
			BASE_URL,
			groupId.replace('.', '/'),
			artifactId,
			version,
			type
		)

		responses.add(responses.POST, 'http://hostname:9990/management', json={"result": {'BYTES_VALUE': 'asdf'}}, status=200)
		responses.add(responses.HEAD, url, json={"uploaed": True}, status=200)
		client.deploy(groupId, artifactId, version, type, server_groups, path, enabled, force, content_host,
		              content_host_ep, content_host_port)


		version = 'version-SNAPSHOT'

		BASE_URL = '{}://{}:{}/{}/service/local/artifact' \
		           '/maven/content?r=public'.format(scheme,
		                                            content_host,
		                                            content_host_port,
		                                            content_host_ep)

		url = '{0}&g={1}&a={2}&v={3}&p={4}'.format(
			BASE_URL,
			groupId.replace('.', '/'),
			artifactId,
			version,
			type
		)

		responses.add(responses.HEAD, url, json={"uploaed": True}, status=200)

		# client.deploy(groupId, artifactId, version, type, server_groups, path, enabled, force, content_host,
		#               content_host_ep, content_host_port)


		# path is None, end_point is maven2
		content_host_ep = 'maven2'
		BASE_URL = '{}://{}:{}/{}'.format(scheme,
		                                  content_host,
		                                  content_host_port,
		                                  content_host_ep)
		url = '{0}/{1}/{2}/{3}/{2}-{3}.{4}'.format(
			BASE_URL,
			groupId.replace('.', '/'),
			artifactId,
			version,
			type)

		responses.add(responses.HEAD, url, json={"uploaed": True}, status=200)
		client.deploy(groupId, artifactId, version, type, server_groups, path, enabled, force, content_host,
		              content_host_ep, content_host_port)

		# path is None, end_pint is None
		content_host_ep = None
		BASE_URL = '{}://{}:{}/service/local/repositories/releases/content'.format(scheme,
		                                                                           content_host,
		                                                                           content_host_port)
		url = '{0}/{1}/{2}/{3}/{2}-{3}.{4}'.format(
			BASE_URL,
			groupId.replace('.', '/'),
			artifactId,
			version,
			type
		)

		responses.add(responses.HEAD, url, json={"uploaed": True}, status=200)
		client.deploy(groupId, artifactId, version, type, server_groups, path, enabled, force, content_host,
		              content_host_ep, content_host_port)

		type = 'jar'
		url = '{0}/{1}/{2}/{3}/{2}-{3}.{4}'.format(
			BASE_URL,
			groupId.replace('.', '/'),
			artifactId,
			version,
			type
		)
		responses.add(responses.HEAD, url, json={"uploaed": True}, status=200)
		client.deploy(groupId, artifactId, version, type, server_groups, path, enabled, force, content_host,
		              content_host_ep, content_host_port)

		# path is not None
		path = 'path'

		# path not exist exception catching
		url = client.endpoint + '/add-content'
		responses.add(responses.POST, url, json={"result": {'BYTES_VALUE': 'asdf'}}, status=200)
		client.deploy(groupId, artifactId, version, type, server_groups, path, enabled, force, content_host,
		              content_host_ep, content_host_port)



	@responses.activate
	def test_disalbe_success(self):
		client = Client('hostname')
		responses.add(responses.POST, client.endpoint, json={"outcome": "success"}, status=200)
		client.disable('name', 'server_group')

	@responses.activate
	def test_disalbe_fail(self):
		client = Client('hostname')
		responses.add(responses.POST, client.endpoint, json={"outcome": "fail"}, status=300)
		client.disable('name', 'server_group')

	@responses.activate
	def test_enable_success(self):
		client = Client('hostname')
		responses.add(responses.POST, client.endpoint, json={"outcome": "success"}, status=200)
		client.enable('name', 'server_group')

	@responses.activate
	def test_enable_fail(self):
		client = Client('hostname')
		responses.add(responses.POST, client.endpoint, json={"outcome": "fail"}, status=300)
		client.enable('name', 'server_group')

	@responses.activate
	def test_undeploy_success(self):
		client = Client('hostname')
		responses.add(responses.POST, client.endpoint, json={"outcome": "success"}, status=200)
		reps = client.undeploy("name", "A")

		assert reps.json() == {"outcome": "success"}